      SUBROUTINE SETMS_KORALW_79
     %   (KEYALL,IGAUGE,DGAUGE,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
************************************************************************
! Overwrites GRACE initialisation according to KORALW needs
! KEYALL = 100*IWEAK +10*IHIGGS +IBACKGR
!   ibackgr = 0  - doubly resonant W-pairs
!           = 1  - all 4-fermion
!   ihiggs  = 0  - no higgs
!           = 1  - with higgs (watch out for mass set-up !!)
!   iweak   = 0  - no weak (not sure what that means...)
!           = 1  - with weak
!   gauges
!   - unitary
!       igauge(1,..,4) = 0 
!       dgauge(1,..,4) = anything (re-set to 1d20 anyway)
!   - covariant
!       igauge(1,..,4) = 1,..,4 
!       dgauge(1,..,4) = anything (eg, 1,..,4) 
************************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION  XPAR (100),NPAR (100) 
      DIMENSION  AMAFIN(20), DGAUGE(4), IGAUGE(4)

!************ GRACE stuff ******************
      INCLUDE 'incl1_79.f'
      INCLUDE 'inclk_79.f'
      COMMON /A79_AMSPIN/JHS(NEXTRN), JHE(NEXTRN), ASPIN
!************ end GRACE stuff ******************
*-----------------------------------------------------------------------

!************ KORALW stuff ******************
      ALFWIN  = XPAR(3)
      ALPHAW  = 1D0/ ALFWIN
      XAMAZ   = XPAR(4)
      GAMMZ   = XPAR(5)
      XAMAW   = XPAR(6)
      GAMMW   = XPAR(7)
!************ end KORALW stuff ******************

      ibackgr = MOD(KeyAll,10)
      ihiggs  = MOD(KeyAll,100)/10
      iweak   = MOD(KeyAll,1000)/100

* Constants

      GEVPB = GPICOB
      ALPHA = ALPHAW

*
! Signals:  mnBudB 2,3,15 (81)
! Signals:  enBudB 2,3,26 (71)
! Signals:  mnBeBn 40,41,63 (87)
! Signals:  enBeBn 84,85,117 (77)

! Signals:  uBdmBn 2,3,25
! Signals:  uBdeBn 46,47,69
      IF(IBACKGR.EQ.0) THEN
* Graph selection (1) -- W-pairs
        DO NG = 1, NGRAPH
          JSELG(NG) = 0
        ENDDO
        JSELG(    1 ) = 1
        JSELG(    2 ) = 1
        JSELG(    3 ) = 1
        JSELG(   35 ) = 1
      ELSE
* Graph selection (1) -- 4-fermion
        DO NG = 1, NGRAPH
          JSELG(NG) = 1
        ENDDO
      ENDIF
*-----------------------------------------------------------------------
* Mass                                                                  
      AMWB = XAMAW                                                   
      AMZB = XAMAZ                                                   
      AMAB =  0.0D0                                                     
                                                                        
      AMXB =   AMWB                                                     
      AMX3 =   AMZB                                                     
                                                                        
      AMPH = 2000.0D0                                                   
                                                                        
      AMNE = AMAFIN(12)
      AMNM = AMAFIN(14)
      AMNT = AMAFIN(16) 
      AMLU = AMNE                                                       
                                                                        
      AMEL = AMAFIN(11)
      AMMU = AMAFIN(13)
      AMTA = AMAFIN(15)
      AMLD = AMEL                                                       
                                                                        
      AMUQ = AMAFIN(2)
      AMCQ = AMAFIN(4)
      AMTQ = 174.0D0                                                    
      AMQU = AMUQ                                                       
                                                                        
      AMDQ = AMAFIN(1)
      AMSQ = AMAFIN(3)
      AMBQ = AMAFIN(5)
      AMQD = AMDQ                                                       
*                                                                       
* Width                                                                 
      AGWB = GAMMW                                                    
      AGZB = GAMMZ                                                    
                                                                        
      AGXB = AGWB                                                       
      AGX3 = AGZB                                                       
                                                                        
*
* Gauge parameters
      IGAUAB = IGAUGE(1)
      IGAUWB = IGAUGE(2)
      IGAUZB = IGAUGE(3)
      IGAUGL = IGAUGE(4)

      AGAUGE(IGAUAB) = DGAUGE(1)
      AGAUGE(IGAUWB) = DGAUGE(2)
      AGAUGE(IGAUZB) = DGAUGE(3)
      AGAUGE(IGAUGL) = DGAUGE(4)

      AGAUGE(0) = 1.0D20   ! this is the UNITARY setting 
*
* Spin average
      ASPIN = 1.0D0
*           1:  EL-  INITIAL   LPRTCL  MASS=AMEL
      JHS(   1) = 0
      JHE(   1) = LEXTRN - 1
      ASPIN = ASPIN/DBLE(JHE(   1)-JHS(   1)+1)
*           2:  EL+  INITIAL   LANTIP  MASS=AMEL
      JHS(   2) = 0
      JHE(   2) = LEXTRN - 1
      ASPIN = ASPIN/DBLE(JHE(   2)-JHS(   2)+1)
*           3:  MU-  FINAL     LPRTCL  MASS=AMMU
      JHS(   3) = 0
      JHE(   3) = LEXTRN - 1
*           4:  NM   FINAL     LANTIP  MASS=AMNM
      JHS(   4) = 0
      JHE(   4) = LEXTRN - 1
*           5:  UQ   FINAL     LPRTCL  MASS=AMUQ
      JHS(   5) = 0
      JHE(   5) = LEXTRN - 1
*           6:  DQ+  FINAL     LANTIP  MASS=AMDQ
      JHS(   6) = 0
      JHE(   6) = LEXTRN - 1
*
* Graph selection (2)
      JWEAKB = IWEAK
      JHIGGS = IHIGGS

      RETURN
      END

