
! *****************************************************
      SUBROUTINE AMP4F( Q1,IFLBM1, Q2,IFLBM2
     $      ,P1,IFLAV1, P2,IFLAV2, P3,IFLAV3, P4,IFLAV4
     $      , WTMOD4F,WT4F )
! *****************************************************
! external 4fermion matrix elements calculations
! INPUTS
!   q1,q2              - beam 4momenta (e-, e+)
!   iflbm1,iflbm2      - beam IDs
!   p1..p4             - final state fermions 4momenta
!                        p1:    fermion of W-
!                        p2:antifermion of W-
!                        p3:    fermion of W+
!                        p4:antifermion of W+
!   iflav1..iflav4     - final state fermions IDs
! OUTPUTS
!   wtmod4f        - principal weight for rejection
!   wt4f(9)        - auxiliary weights wector
! *****************************************************
! INTERNAL parameters: KEYALL, IGAUGE, DGAUGE
! KEYALL = 100*IWEAK +10*IHIGGS +IBACKGR
!   ibackgr = 0  - doubly resonant W-pairs
!           = 1  - all 4-fermion
!   ihiggs  = 0  - no higgs
!           = 1  - with higgs (watch out for mass set-up !!)
!   iweak   = 0  - no weak (not sure what that means...)
!           = 1  - with weak
!   gauges
!   - unitary
!       igauge(1,..,4) = 0 
!       dgauge(1,..,4) = anything (re-set to 1d20 anyway)
!   - covariant
!       igauge(1,..,4) = 1,..,4 
!       dgauge(1,..,4) = anything (eg, 1,..,4) 
*******************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)  

! parameters for 4fermion amplitudes
      COMMON /AM4PAR/ NPAR(100),XPAR(100),AMAFIN(20),SIN2W,GPICOB

      
      PARAMETER ( MXDIM = 50 )
      REAL*8  X(MXDIM), Y(MXDIM)
      DIMENSION Q1(4),Q2(4),P1(4),P2(4),P3(4),P4(4)
      DIMENSION WT4F(9), ICOD(9)
      DIMENSION IG_UNI(4), IG_COV(4), DG_UNI(4), DG_COV(4)

      save

! setups - gauge
!   unitary
      ig_uni(1) = 0
      ig_uni(2) = 0
      ig_uni(3) = 0
      ig_uni(4) = 0

      dg_uni(1) = 1d0
      dg_uni(2) = 1d0
      dg_uni(3) = 1d0
      dg_uni(4) = 1d0

!   covariant
      ig_cov(1) = 1
      ig_cov(2) = 2
      ig_cov(3) = 3
      ig_cov(4) = 4

      dg_cov(1) = 2d0
      dg_cov(2) = 3d0
      dg_cov(3) = 4d0
      dg_cov(4) = 5d0
! other setups
      ihiggs = 0
      iweak = 1

      ibac = 100*iweak +10*ihiggs + 1
      isig = 100*iweak +10*ihiggs + 0


*           1:  EL-  INITIAL   LPRTCL  MASS=AMEL
      x( 1) = q1(1)
      x( 2) = q1(2)
      x( 3) = q1(3)
      x( 4) = q1(4)
*           2:  EL+  INITIAL   LANTIP  MASS=AMEL
      x( 5) = q2(1)
      x( 6) = q2(2)
      x( 7) = q2(3)
      x( 8) = q2(4)
*           3:  MU-  FINAL     LPRTCL  MASS=AMMU
      x(11) = p1(1)
      x(12) = p1(2)
      x(13) = p1(3)
      x(14) = p1(4)
*           4:  NM   FINAL     LANTIP  MASS=AMNM
      x(21) = p2(1)
      x(22) = p2(2)
      x(23) = p2(3)
      x(24) = p2(4)
*           5:  UQ   FINAL     LPRTCL  MASS=AMUQ
      x(31) = p3(1)
      x(32) = p3(2)
      x(33) = p3(3)
      x(34) = p3(4)
*           6:  DQ+  FINAL     LANTIP  MASS=AMDQ
      x(41) = p4(1)
      x(42) = p4(2)
      x(43) = p4(3)
      x(44) = p4(4)


! ==== for symmetry
*           1:  EL-  INITIAL   LPRTCL  MASS=AMEL
      y( 1) = -q2(1)
      y( 2) = -q2(2)
      y( 3) = -q2(3)
      y( 4) = q2(4)
*           2:  EL+  INITIAL   LANTIP  MASS=AMEL
      y( 5) = -q1(1)
      y( 6) = -q1(2)
      y( 7) = -q1(3)
      y( 8) = q1(4)
*           3:  MU-  FINAL     LPRTCL  MASS=AMMU
      y(11) = -p4(1)
      y(12) = -p4(2)
      y(13) = -p4(3)
      y(14) = p4(4)
*           4:  NM   FINAL     LANTIP  MASS=AMNM
      y(21) = -p3(1)
      y(22) = -p3(2)
      y(23) = -p3(3)
      y(24) = p3(4)
*           5:  UQ   FINAL     LPRTCL  MASS=AMUQ
      y(31) = -p2(1)
      y(32) = -p2(2)
      y(33) = -p2(3)
      y(34) = p2(4)
*           6:  DQ+  FINAL     LANTIP  MASS=AMDQ
      y(41) = -p1(1)
      y(42) = -p1(2)
      y(43) = -p1(3)
      y(44) = p1(4)

!============ decoding final states ===============
! Codes for final state flavors
        icod(1)=102
        icod(2)=104
        icod(3)=302
        icod(4)=304
        icod(5)=502
        icod(6)=504
        icod(7)=1112
        icod(8)=1314
        icod(9)=1516

! make final state codes
        icdm =  iflav1*100 -iflav2
        icdp = -iflav4*100 +iflav3
! internal convention of KORALW
        im = 0
        ip = 0
        do ii=1,9
          if(icod(ii).eq.icdm) im = ii
          if(icod(ii).eq.icdp) ip = ii
        enddo

      if(im.eq.0 .or. ip.eq.0) then
        write(6,*) 'amp4f ==> wrong PDG codes'
        stop
      endif

      ichnum = 10*im+ip

! Switched to KORALW internal function ELMATR  
        imode=1
        sig_kor = elmatr(imode)

      if(ichnum.eq.0) then
!     ====================
        write(6,*) 'amp4f ==> wrong PDG codes'
        stop

      elseif(ichnum.eq.11) then
*     ======
        CALL SETMS_KORALW_11
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        bac_uni = func_11(x)
        write(11,'(a,f18.11)') '11 wt4f=',bac_uni/sig_kor
!        CALL SETMS_KORALW_11
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        bac_cov = func_11(x)
!        CALL SETMS_KORALW_11
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        sig_grc = func_11(x)
      elseif(ichnum.eq.14.or.ichnum.eq.41) then
*     ======
        CALL SETMS_KORALW_14
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.14) then
          bac_uni = func_14(x)
          write(14,'(a,f18.11)') '14 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_14(y)
          write(41,'(a,f18.11)') '41 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_14
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.14) then
!          bac_cov = func_14(x)
!        else
!          bac_cov = func_14(y)
!        endif
!        CALL SETMS_KORALW_14
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.14) then
!          sig_grc = func_14(x)
!        else
!          sig_grc = func_14(y)
!        endif
      elseif(ichnum.eq.17.or.ichnum.eq.71) then
*     ======
        CALL SETMS_KORALW_17
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.17) then
          bac_uni = func_17(x)
          write(17,'(a,f18.11)') '17 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_17(y)
          write(71,'(a,f18.11)') '71 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_17
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.17) then
!          bac_cov = func_17(x)
!        else
!          bac_cov = func_17(y)
!        endif
!        CALL SETMS_KORALW_17
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.17) then
!          sig_grc = func_17(x)
!        else
!          sig_grc = func_17(y)
!        endif
      elseif(ichnum.eq.18.or.ichnum.eq.81) then
*     ======
        CALL SETMS_KORALW_18
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.18) then
          bac_uni = func_18(x)
          write(18,'(a,f18.11)') '18 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_18(y)
          write(81,'(a,f18.11)') '81 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_18
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.18) then
!          bac_cov = func_18(x)
!        else
!          bac_cov = func_18(y)
!        endif
!        CALL SETMS_KORALW_18
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.18) then
!          sig_grc = func_18(x)
!        else
!          sig_grc = func_18(y)
!        endif
      elseif(ichnum.eq.19.or.ichnum.eq.91) then
*     ======
        CALL SETMS_KORALW_19
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.19) then
          bac_uni = func_19(x)
          write(19,'(a,f18.11)') '19 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_19(y)
          write(91,'(a,f18.11)') '91 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_19
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.19) then
!          bac_cov = func_19(x)
!        else
!          bac_cov = func_19(y)
!        endif
!        CALL SETMS_KORALW_19
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.19) then
!          sig_grc = func_19(x)
!        else
!          sig_grc = func_19(y)
!        endif
      elseif(ichnum.eq.44) then
*     ======
        CALL SETMS_KORALW_44
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        bac_uni = func_44(x)
        write(44,'(a,f18.11)') '44 wt4f=',bac_uni/sig_kor
!        CALL SETMS_KORALW_44
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        bac_cov = func_44(x)
!        CALL SETMS_KORALW_44
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        sig_grc = func_44(x)
      elseif(ichnum.eq.47.or.ichnum.eq.74) then
*     ======
        CALL SETMS_KORALW_47
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.47) then
          bac_uni = func_47(x)
          write(47,'(a,f18.11)') '47 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_47(y)
          write(74,'(a,f18.11)') '74 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_47
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.47) then
!          bac_cov = func_47(x)
!        else
!          bac_cov = func_47(y)
!        endif
!        CALL SETMS_KORALW_47
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.47) then
!          sig_grc = func_47(x)
!        else
!          sig_grc = func_47(y)
!        endif
      elseif(ichnum.eq.48.or.ichnum.eq.84) then
*     ======
        CALL SETMS_KORALW_48
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.48) then
          bac_uni = func_48(x)
          write(48,'(a,f18.11)') '48 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_48(y)
          write(84,'(a,f18.11)') '84 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_48
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.48) then
!          bac_cov = func_48(x)
!        else
!          bac_cov = func_48(y)
!        endif
!        CALL SETMS_KORALW_48
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.48) then
!          sig_grc = func_48(x)
!        else
!          sig_grc = func_48(y)
!        endif
      elseif(ichnum.eq.49.or.ichnum.eq.94) then
*     ======
        CALL SETMS_KORALW_49
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.49) then
          bac_uni = func_49(x)
          write(49,'(a,f18.11)') '49 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_49(y)
          write(94,'(a,f18.11)') '94 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_49
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.49) then
!          bac_cov = func_49(x)
!        else
!          bac_cov = func_49(y)
!        endif
!        CALL SETMS_KORALW_49
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.49) then
!          sig_grc = func_49(x)
!        else
!          sig_grc = func_49(y)
!        endif
      elseif(ichnum.eq.77) then
*     ======
        CALL SETMS_KORALW_77
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        bac_uni = func_77(x)
        write(77,'(a,f18.11)') '77 wt4f=',bac_uni/sig_kor
!        CALL SETMS_KORALW_77
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        bac_cov = func_77(x)
!        CALL SETMS_KORALW_77
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        sig_grc = func_77(x)
      elseif(ichnum.eq.78.or.ichnum.eq.87) then
*     ======
        CALL SETMS_KORALW_78
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.78) then
          bac_uni = func_78(x)
          write(78,'(a,f18.11)') '78 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_78(y)
          write(87,'(a,f18.11)') '87 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_78
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.78) then
!          bac_cov = func_78(x)
!        else
!          bac_cov = func_78(y)
!        endif
!        CALL SETMS_KORALW_78
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.78) then
!          sig_grc = func_78(x)
!        else
!          sig_grc = func_78(y)
!        endif
      elseif(ichnum.eq.79.or.ichnum.eq.97) then
*     ======
        CALL SETMS_KORALW_79
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.79) then
          bac_uni = func_79(x)
          write(79,'(a,f18.11)') '79 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_79(y)
          write(97,'(a,f18.11)') '97 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_79
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.79) then
!          bac_cov = func_79(x)
!        else
!          bac_cov = func_79(y)
!        endif
!        CALL SETMS_KORALW_79
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.79) then
!          sig_grc = func_79(x)
!        else
!          sig_grc = func_79(y)
!        endif
      elseif(ichnum.eq.88) then
*     ======
        CALL SETMS_KORALW_88
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        bac_uni = func_88(x)
        write(88,'(a,f18.11)') '88 wt4f=',bac_uni/sig_kor
!        CALL SETMS_KORALW_88
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        bac_cov = func_88(x)
!        CALL SETMS_KORALW_88
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        sig_grc = func_88(x)
      elseif(ichnum.eq.89.or.ichnum.eq.98) then
*     ======
        CALL SETMS_KORALW_89
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        if(ichnum.eq.89) then
          bac_uni = func_89(x)
          write(89,'(a,f18.11)') '89 wt4f=',bac_uni/sig_kor
        else
          bac_uni = func_89(y)
          write(98,'(a,f18.11)') '98 wt4f=',bac_uni/sig_kor
        endif
!        CALL SETMS_KORALW_89
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.89) then
!          bac_cov = func_89(x)
!        else
!          bac_cov = func_89(y)
!        endif
!        CALL SETMS_KORALW_89
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        if(ichnum.eq.89) then
!          sig_grc = func_89(x)
!        else
!          sig_grc = func_89(y)
!        endif
      elseif(ichnum.eq.99) then
*     ======
        CALL SETMS_KORALW_99
     $   (ibac,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
        bac_uni = func_99(x)
        write(99,'(a,f18.11)') '99 wt4f=',bac_uni/sig_kor
!        CALL SETMS_KORALW_99
!     $   (ibac,ig_cov,dg_cov,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        bac_cov = func_99(x)
!        CALL SETMS_KORALW_99
!     $   (isig,ig_uni,dg_uni,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
!        sig_grc = func_99(x)
      else
!     ====
        sig_kor = 1d0
        bac_uni = 1d0
        sig_grc = 1d0
        bac_cov = 1d0
      endif
!     ====

      DO I4F=1,9
        WT4F(I4F) = 0D0
      ENDDO

      WT4F(1) = sig_kor
      WT4F(2) = bac_uni
      WT4F(3) = sig_grc
      WT4F(4) = bac_cov


      WTMOD4F = bac_uni/sig_kor
!      write(6,*) '4ferm_sign=',sig_kor/sig_grc
!      write(6,*) '4ferm_gaug=',bac_uni/bac_cov
!      write(6,*) 'wtmod4ferm=',wtmod4f
!      write(6,*) '===================='

      END

      subroutine spdetx(ireco)
! empty
      end

