! *****************************************************
      SUBROUTINE AMP4F( Q1,IFLBM1, Q2,IFLBM2
     $      ,P1,IFLAV1, P2,IFLAV2, P3,IFLAV3, P4,IFLAV4
     $      , WTMOD4F,WT4F )
! *****************************************************
! external 4fermion matrix elements calculations
! INPUTS
!   q1,q2              - beam 4momenta (e-, e+)
!   iflbm1,iflbm2      - beam IDs
!   p1..p4             - final state fermions 4momenta
!                        p1:    fermion of W-
!                        p2:antifermion of W-
!                        p3:    fermion of W+
!                        p4:antifermion of W+
!   iflav1..iflav4     - final state fermions IDs
! OUTPUTS
!   wtmod4f        - principal weight for rejection
!   wt4f(9)        - auxiliary weights wector
! *****************************************************
! INTERNAL parameters: KEYALL, IGAUGE, DGAUGE
! KEYALL = 100*IWEAK +10*IHIGGS +IBACKGR
!   ibackgr = 0  - doubly resonant W-pairs
!           = 1  - all 4-fermion
!   ihiggs  = 0  - no higgs
!           = 1  - with higgs (watch out for mass set-up !!)
!   iweak   = 0  - no weak (not sure what that means...)
!           = 1  - with weak
!   gauges
!   - unitary
!       igauge(1,..,4) = 0 
!       dgauge(1,..,4) = anything (re-set to 1d20 anyway)
!   - covariant
!       igauge(1,..,4) = 1,..,4 
!       dgauge(1,..,4) = anything (eg, 1,..,4) 
*******************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)  

! parameters for 4fermion amplitudes
      COMMON /AM4PAR/ NPAR(100),XPAR(100),AMAFIN(20),SIN2W,GPICOB

      
      PARAMETER ( MXDIM = 50 )
      REAL*8  X(MXDIM)
      DIMENSION Q1(4),Q2(4),P1(4),P2(4),P3(4),P4(4)
      DIMENSION WT4F(9), ICOD(9)
      DIMENSION IG_UNI(4), IG_COV(4), DG_UNI(4), DG_COV(4)

      save

! setups - gauge
!   unitary
      ig_uni(1) = 0
      ig_uni(2) = 0
      ig_uni(3) = 0
      ig_uni(4) = 0

      dg_uni(1) = 1d0
      dg_uni(2) = 1d0
      dg_uni(3) = 1d0
      dg_uni(4) = 1d0

!   covariant
      ig_cov(1) = 1
      ig_cov(2) = 2
      ig_cov(3) = 3
      ig_cov(4) = 4

      dg_cov(1) = 2d0
      dg_cov(2) = 3d0
      dg_cov(3) = 4d0
      dg_cov(4) = 5d0
! other setups
      ihiggs = 0
      iweak = 1

      ibac = 100*iweak +10*ihiggs + 1
      isig = 100*iweak +10*ihiggs + 0


*           1:  EL-  INITIAL   LPRTCL  MASS=AMEL
      x( 1) = q1(1)
      x( 2) = q1(2)
      x( 3) = q1(3)
      x( 4) = q1(4)
*           2:  EL+  INITIAL   LANTIP  MASS=AMEL
      x( 5) = q2(1)
      x( 6) = q2(2)
      x( 7) = q2(3)
      x( 8) = q2(4)
*           3:  MU-  FINAL     LPRTCL  MASS=AMMU
      x(11) = p1(1)
      x(12) = p1(2)
      x(13) = p1(3)
      x(14) = p1(4)
*           4:  NM   FINAL     LANTIP  MASS=AMNM
      x(21) = p2(1)
      x(22) = p2(2)
      x(23) = p2(3)
      x(24) = p2(4)
*           5:  UQ   FINAL     LPRTCL  MASS=AMUQ
      x(31) = p3(1)
      x(32) = p3(2)
      x(33) = p3(3)
      x(34) = p3(4)
*           6:  DQ+  FINAL     LANTIP  MASS=AMDQ
      x(41) = p4(1)
      x(42) = p4(2)
      x(43) = p4(3)
      x(44) = p4(4)


!============ decoding final states ===============
! Codes for final state flavors
        icod(1)=102
        icod(2)=104
        icod(3)=302
        icod(4)=304
        icod(5)=502
        icod(6)=504
        icod(7)=1112
        icod(8)=1314
        icod(9)=1516

! make final state codes
        icdm =  iflav1*100 -iflav2
        icdp = -iflav4*100 +iflav3
! internal convention of KORALW
        im = 0
        ip = 0
        do ii=1,9
          if(icod(ii).eq.icdm) im = ii
          if(icod(ii).eq.icdp) ip = ii
        enddo

      if(im.eq.0 .or. ip.eq.0) then
        write(6,*) 'amp4f ==> wrong PDG codes'
        stop
      endif

      ichnum = 10*im+ip

! Switched to KORALW internal function ELMATR  
        imode=1
        sig_kor = elmatr(imode)

      if(ichnum.eq.0) then
!     ====================
        write(6,*) 'amp4f ==> wrong PDG codes'
        stop
