
      SUBROUTINE AMPINI(XPAR,NPAR)
! ********************************
! Initialization of the external 4fermion matrix el. codes
! XPAR(100),NPAR(100): input parameter matrices of KORALW as explained in
!                      the manual
! For the moment only GRACE is interfaced
! ********************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION  XPAR (100),NPAR (100) 
      DIMENSION AMAFIN(20) 

! parameters for 4-fermion amplitudes
      COMMON /AM4PAR/ INPAR(100),YXPAR(100),YAMAFIN(20),YSIN2W,YGPICOB

! additional information can be obtained with the help of routine
      CALL MASOW(SIN2W,GPICOB,AMAFIN)
!     where sin2w is a sine of weinberg angle 
!     and   AMAFIN contained masses of W decay products
!     all as printed by the KORALW
!

! filling /AM4PAR/
      do ii=1,100
        inpar(ii) = npar(ii)
        yxpar(ii) = xpar(ii)
      enddo
      do ii=1,20
        yamafin(ii) = amafin(ii)
      enddo
      ygpicob =gpicob
      ysin2w = sin2w
! end  filling /AM4PAR/

      WRITE(6,'(10X,A)')
     $         '*******************************************************'
      WRITE(6,'(10X,A)')
     $         '****** AMPINI: external library initialization ********'
      WRITE(6,'(10X,A)')
     $         '*******************************************************'
      WRITE(6,*) ' '

! here users code should come

      WRITE(6,'(10X,A)') 
     $         '*******************************************************'
      WRITE(6,'(10X,A)') 
     $         '************* AMPINI -- GRACE activation **************'
      WRITE(6,'(10X,A/)') 
     $         '*******************************************************'
      WRITE(*,'(20X,A/)') 'GRACE   Ver.   1. 1'
      WRITE(*,'(10X,A/)')
     .'(c)Copyright 1990-1994 Minami-Tateya Group (KEK, Japan)     '
      WRITE(6,'(10X,A)') 
     $         '*******************************************************'

      KeyPhy = NPAR(2)
      KeyZet = MOD(KeyPhy,1000)/100
      KeyWu  = MOD(KeyPhy,1000000)/100000

      if( keyzet.ne.1 ) then
        write(6,*)'AMPINI==> Sorry, not implemented: KeyZet =',keyzet
!        stop
      endif
      if( keywu.ne.1 ) then
        write(6,*)'AMPINI==> Sorry, not implemented: KeyWu =',keywu
!        stop
      endif
      call gr_init(1,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
         
      END



! *****************************************************
      SUBROUTINE AMP4F( Q1,IFLBM1, Q2,IFLBM2
     $      ,P1,IFLAV1, P2,IFLAV2, P3,IFLAV3, P4,IFLAV4
     $      , WTMOD4F,WT4F )
! *****************************************************
! external 4-fermion matrix elements calculations
! INPUTS
!   q1,q2              - beam 4momenta (e-, e+)
!   iflbm1,iflbm2      - beam IDs
!   p1..p4             - final state fermions 4momenta
!                        p1:    fermion of W-
!                        p2:antifermion of W-
!                        p3:    fermion of W+
!                        p4:antifermion of W+
!   iflav1..iflav4     - final state fermions IDs
!   WARNING: for the moment iflav-s are dummy, and 4momenta must be
!            ordered as described above
! OUTPUTS
!   wtmod4f        - principal weight for rejection
!   wt4f(9)        - auxiliary weights wector
!      wt4f(1) = elmatr_WWpairs
!      wt4f(2) = elmatr_4fermions
!      wtmod4f = elmatr_4fermions/elmatr_WWpairs
! *****************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)  

! parameters for 4fermion amplitudes
      COMMON /AM4PAR/ NPAR(100),XPAR(100),AMAFIN(20),SIN2W,GPICOB

      
      PARAMETER ( MXDIM = 50 )
      REAL*8  X(MXDIM) 
      DIMENSION Q1(4),Q2(4),P1(4),P2(4),P3(4),P4(4)
      DIMENSION WT4F(9)

      save

*           1:  EL-  INITIAL   LPRTCL  MASS=AMEL
      x( 1) = q1(1)
      x( 2) = q1(2)
      x( 3) = q1(3)
      x( 4) = q1(4)
*           2:  EL+  INITIAL   LANTIP  MASS=AMEL
      x( 5) = q2(1)
      x( 6) = q2(2)
      x( 7) = q2(3)
      x( 8) = q2(4)
*           3:  MU-  FINAL     LPRTCL  MASS=AMMU
      x(11) = p1(1)
      x(12) = p1(2)
      x(13) = p1(3)
      x(14) = p1(4)
*           4:  NM   FINAL     LANTIP  MASS=AMNM
      x(21) = p2(1)
      x(22) = p2(2)
      x(23) = p2(3)
      x(24) = p2(4)
*           5:  UQ   FINAL     LPRTCL  MASS=AMUQ
      x(31) = p3(1)
      x(32) = p3(2)
      x(33) = p3(3)
      x(34) = p3(4)
*           6:  DQ+  FINAL     LANTIP  MASS=AMDQ
      x(41) = p4(1)
      x(42) = p4(2)
      x(43) = p4(3)
      x(44) = p4(4)

      DO I4F=1,9
        WT4F(I4F) = 0D0
      ENDDO

!!      WTMOD4F = 1d0
!!      if(iflav1.eq.13.and.iflav3.eq.2) then  !(81)
!!      if(iflav1.eq.1.and.iflav3.eq.2) then  !(11)
!!      if(iflav1.eq.1.and.iflav3.eq.14) then  !(18)
!!      if(iflav1.eq.13.and.iflav3.eq.14) then  !(88)
*=======================================================================
*==== 4-fermions
      CALL SM_KORALW(1,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
      el_4ferm = func(x)
*=======================================================================
*==== W-pairs
      CALL SM_KORALW(0,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
      el_WWpair = func(x)
*=======================================================================


      WT4F(1) = el_WWpair
      WT4F(2) = el_4ferm

      WTMOD4F = el_4ferm/el_WWpair
!!      write(8,'(a,f18.11)') '88 wt4f=',wtmod4f
!!      ENDIF

cc      write(6,*) '4ferm=',el_WWpair,el_4ferm
cc      write(6,*) 'wtmod4ferm=',wtmod4f

      END

      END
      subroutine spdetx(ireco)
! empty
      end





