      SUBROUTINE SM_KORALW(IBACKGR,XPAR,NPAR,SIN2W,GPICOB,AMAFIN)
************************************************************************
* Overwrites GRACE initialisation according to KORALW needs
* ibackgr = 0  - doubly resonant W-pairs  
* ibackgr = 1  - complete 4fermion process
************************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION  XPAR ( *),NPAR ( *) 
      DIMENSION  AMAFIN(20) 

!************ GRACE stuff ******************
      INCLUDE 'incl1.f'
      INCLUDE 'inclk.f'
      COMMON /AMSPIN/JHS(NEXTRN), JHE(NEXTRN), ASPIN
!************ end GRACE stuff ******************
*-----------------------------------------------------------------------

!************ KORALW stuff ******************
      ALFWIN  = XPAR(3)
      ALPHAW  = 1D0/ ALFWIN
      XAMAZ   = XPAR(4)
      GAMMZ   = XPAR(5)
      XAMAW   = XPAR(6)
      GAMMW   = XPAR(7)
!************ end KORALW stuff ******************

* Constants

      GEVPB = GPICOB
      ALPHA = ALPHAW

*
! Signals:  mnud 2,3,15 (81)
! Signals:  enen 84,85,117
! Signals:  (11) 110, 111, 112, 124
! Signals:  (88) 23, 24, 25, 47
! Signals:  (18) 1, 2, 3, 25
      IF(IBACKGR.EQ.0) THEN
* Graph selection (1) -- W-pairs
        DO NG = 1, NGRAPH
          JSELG(NG) = 0
        ENDDO
        JSELG( 2) = 1
        JSELG( 3) = 1
        JSELG(15) = 1
      ELSE
* Graph selection (1) -- 4-fermion
        DO NG = 1, NGRAPH
          JSELG(NG) = 1
        ENDDO
      ENDIF
*-----------------------------------------------------------------------
* Mass                                                                  
      AMWB = XAMAW                                                   
      AMZB = XAMAZ                                                   
      AMAB =  0.0D0                                                     
                                                                        
      AMXB =   AMWB                                                     
      AMX3 =   AMZB                                                     
                                                                        
      AMPH = 2000.0D0                                                   
                                                                        
      AMNE = AMAFIN(12)
      AMNM = AMAFIN(14)
      AMNT = AMAFIN(16) 
      AMLU = AMNE                                                       
                                                                        
      AMEL = AMAFIN(11)
      AMMU = AMAFIN(13)
      AMTA = AMAFIN(15)
      AMLD = AMEL                                                       
                                                                        
      AMUQ = AMAFIN(2)
      AMCQ = AMAFIN(4)
      AMTQ = 174.0D0                                                    
      AMQU = AMUQ                                                       
                                                                        
      AMDQ = AMAFIN(1)
      AMSQ = AMAFIN(3)
      AMBQ = AMAFIN(5)
      AMQD = AMDQ                                                       
*                                                                       
* Width                                                                 
      AGWB = GAMMW                                                    
      AGZB = GAMMZ                                                    
                                                                        
      AGXB = AGWB                                                       
      AGX3 = AGZB                                                       
                                                                        
* Masses of external particles
      AMASS1(   1) = AMEL
      AMASS1(   2) = AMEL
      AMASS1(   3) = AMMU
      AMASS1(   4) = AMNM
      AMASS1(   5) = AMUQ
      AMASS1(   6) = AMDQ
      AMASS2(   1) = AMASS1(   1)**2
      AMASS2(   2) = AMASS1(   2)**2
      AMASS2(   3) = AMASS1(   3)**2
      AMASS2(   4) = AMASS1(   4)**2
      AMASS2(   5) = AMASS1(   5)**2
      AMASS2(   6) = AMASS1(   6)**2
*
* Gauge parameters (default is unitary gauge)
      IGAUAB = 0
      IGAUWB = 0
      IGAUZB = 0
      IGAUGL = 0
      AGAUGE(IGAU00) = 1.0D0
      AGAUGE(IGAUAB) = 1.0D0
      AGAUGE(IGAUWB) = 1.0D0
      AGAUGE(IGAUZB) = 1.0D0
      AGAUGE(IGAUGL) = 1.0D0
*
* Spin average
      ASPIN = 1.0D0
*           1:  EL-  INITIAL   LPRTCL  MASS=AMEL
      JHS(   1) = 0
      JHE(   1) = LEXTRN - 1
      ASPIN = ASPIN/DBLE(JHE(   1)-JHS(   1)+1)
*           2:  EL+  INITIAL   LANTIP  MASS=AMEL
      JHS(   2) = 0
      JHE(   2) = LEXTRN - 1
      ASPIN = ASPIN/DBLE(JHE(   2)-JHS(   2)+1)
*           3:  MU-  FINAL     LPRTCL  MASS=AMMU
      JHS(   3) = 0
      JHE(   3) = LEXTRN - 1
*           4:  NM   FINAL     LANTIP  MASS=AMNM
      JHS(   4) = 0
      JHE(   4) = LEXTRN - 1
*           5:  UQ   FINAL     LPRTCL  MASS=AMUQ
      JHS(   5) = 0
      JHE(   5) = LEXTRN - 1
*           6:  DQ+  FINAL     LANTIP  MASS=AMDQ
      JHS(   6) = 0
      JHE(   6) = LEXTRN - 1
*
* Graph selection (2)
      JWEAKB = 1
      JHIGGS = 0

      RETURN
      END

