      SUBROUTINE koralw(mode,xpar,npar)     
!     *********************************    
! =======================================================================
! =======================================================================
! ============================KORALW=====================================
! =======================WW PAIR PRODUCTION==============================
! ==================INITIAL STATE EXPONENTIATION========================= 
! =======================================================================
! =======================================================================
! =========================VERSION 1.32 =================================
! =======================================================================
! ========================== May 1997 ===================================
! =======================================================================
!     This program is written by: 
!          M. Skrzypek    (skrzypek@hpjmiady.ifj.edu.pl)
!     in collaboration with  
!          S. Jadach      (jadach@cernvm.cern.ch),
!          W. Placzek     (placzek@hephp02.phys.utk.edu),
!          Z. Was         (wasm@cernvm.cern.ch)
!     We acknowledge warmly very usefull help of M. Martinez in testing
!     the program.
! =======================================================================
! =======================================================================
!  
!          
! !!! SOME CONTROLL HISTOGRAMING STILL IN PROGRAM   !!!   
!          
! Generator of Yennie-Frautschi-Suura type         
! with Exponentiated Initial State Bremsstrahlung.
! ********* input    
! mode =-1/0/1/2 defines       
!       initialization/generation/give-xsection/final-report  
! cmsene = centre of mass energy (gev)  
! npar(1)= KeyRad =  1000*KeyCul+100*KeyNLL+10*KeyFSR+KeyISR
!     KeyCul Coulomb correction, 
!     KeyNLL=0 sets Next-to Leading alpha/pi terms to zero
!     KeyNLL=1 alpha/pi in YFS formfactor is kept
!     KeyFSR   Fnitial State Radiation switch, INACTIVE
!     KeyISR=0   Initial State Radiation OFF,
!     KeyISR=1  Initial State Radiation ON.
! npar(2)= KeyPhy
!     KeyPhy = 100000*KeyWu +10000*KeyRed +1000*KeySpn 
!                       +100*KeyZet +10*KeyMas +KeyBra
!     KeyBra= 0 Born branching ratios, no mixing
!     KeyBra= 1 Branching ratios with mixing and QCD
!     KeyMas= 0 Masless kinematics for W decay products 
!     KeyMas= 1 Massive kinematics for W decay products 
!     KeyZet= 0 Z width in Z propagator: s/M_Z *GAMM_Z
!     KeyZet= 1 Z width in Z propagator:   M_Z *GAMM_Z 
!     KeyZet= 2 Z zero width in Z propagator.
!     KeySpn= 0-off, 1-on for  spin effects in W decays 
!     KeyRed= Reduction of massive FS to massles Matr.El.
!     KeyRed= 0 fine
!     KeyRed= 1 crude, 4-mom. non conserving
!     KeyWu= 0 W width in W propagator: s/M_W *GAMM_W
!     KeyWu= 1 W width in W propagator:   M_W *GAMM_W 
!     KeyWu= 2 no (0) W width in W propagator.
! npar(3)= KeyTek= 100*KeySmp +10*KeyRnd +KeyWgt
!     KeyWgt =0, WTMOD=1 useful for apparatus Monte Carlo.
!     KeyWgt =1, WTMOD varying, option faster and safer
!     KeyWgt =2, WTMOD=1 for internal matrix el, AND varying for external
!                                                               matrix el.
!     KeyRnd = ?????
!     KeySmp =0 Presampler set as in KORALW v. 1.02-1.2
!     KeySmp =1 New option, useful for unweightet events far from WW peaks
! npar(4)= KeyMis = 100*KeyAcc +10*Key4f +KeyMix
!     miscelaneus auxiliarym key for tests
!     actualy KeyMix=0 causes  sinw2 to be according LEP200 presription
!     while   KeyMix=1 prowides sinw2 as in 1.00 version.
!     Key4f  = 0, external matrix el. OFF
!     Key4f  = 1, external matrix el. ON
!     KeyAcc = 0, anomalous WWV couplings in internal matr. el. OFF
!     KeyAcc = 1, anomalous WWV couplings in internal matr. el. ON
! npar(5)=KeyDwm  decay channel of W- 
! npar(6)=KeyDwp  decay channel of W+ 
!                 0-all chann. according to br. ratios
!                 1-ud, 2-cd, 3-us, 4-cs, 5-ub, 6-cb, 7-e, 8-mu, 9-tau
! npar(7)=NOUT  output unit number,  
!               if npar(7) .le. 0 then NOUT=16. 
! npar(21)=jak1      TAUOLA, tau of W+ decay channel 
! npar(22)=jak2      TAUOLA, tau of W- decay channel 
! npar(23)=ITDKRC    TAUOLA, radiative corr. in leponic tau decays switch
! npar(24)=IFPHOT    PHOTOS, activation switch
! npar(25)=IFHADM    JETSET, W- hadronisation activation switch
! npar(26)=IFHADP    JETSET, W+ hadronisation activation switch
!               !!!! IFHADM and IFHADP are glued together (for now) !!!!
! xpar(1)=cmsene  =  cms energy  
! xpar(2)=gmu     =  G_Fermi
! xpar(3)=alfwin  =  1/alpha_w  
! xpar(4)=amaz    =  mass  of Z0     
! xpar(5)=gammz   =  width of Z0       
! xpar(6)=amaw    =  mass  of W    
! xpar(7)=gammw   =  width of W       
! xpar(8)=vvmin   =  minimum v-variable (dimesionless) =epsilon 
! xpar(9)=vvmax   =  maximum v-variable   
! xpar(10)=wtmax  =  maximum weight for rejection  
! xpar(11)=amh    =  Higgs mass
! xpar(12)=agh    =  Higgs width
! 
! xpar(21) - xpar(57):
!   g1(1)   = DCMPLX(xpar(21),xpar(31))
!   kap(1)  = DCMPLX(xpar(22),xpar(32))
!   lam(1)  = DCMPLX(xpar(23),xpar(33))
!   g4(1)   = DCMPLX(xpar(24),xpar(34))
!   g5(1)   = DCMPLX(xpar(25),xpar(35))
!   kapt(1) = DCMPLX(xpar(26),xpar(36))
!   lamt(1) = DCMPLX(xpar(27),xpar(37))
!  !----
!   g1(2)   = DCMPLX(xpar(41),xpar(51))
!   kap(2)  = DCMPLX(xpar(42),xpar(52))
!   lam(2)  = DCMPLX(xpar(43),xpar(53))
!   g4(2)   = DCMPLX(xpar(44),xpar(54))
!   g5(2)   = DCMPLX(xpar(45),xpar(55))
!   kapt(2) = DCMPLX(xpar(46),xpar(56))
!   lamt(2) = DCMPLX(xpar(47),xpar(57))
!  where:
!   g1(2)  : g_1^Z, anomalous coupling for WWZ vertex
!   kap(2) : kappa_Z, anomalous coupling for WWZ vertex 
!   lam(2) : lambda_Z, anomalous coupling for WWZ vertex 
!   g4(2)  : g_4^Z, anomalous coupling for WWZ vertex
!   g5(2)  : g_5^Z, anomalous coupling for WWZ vertex
!   kapt(2): kappa-tilde_Z, anomalous coupling for WWZ vertex
!   lamt(2): lambda-tilde_Z, anomalous coupling for WWZ vertex
!  !----
!   g1(1)  : g_1^g, anomalous coupling for WWg vertex
!   kap(1) : kappa_g, anomalous coupling for WWg vertex 
!   lam(1) : lambda_g, anomalous coupling for WWg vertex 
!   g4(1)  : g_4^g, anomalous coupling for WWg vertex
!   g5(1)  : g_5^g, anomalous coupling for WWg vertex
!   kapt(1): kappa-tilde_g, anomalous coupling for WWg vertex
!   lamt(1): lambda-tilde_g, anomalous coupling for WWg vertex


! ********* output   
!
! fourmomenta and photon multiplicity in standard common HEPEVT:  
!
!      COMMON/HEPEVT/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
!     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
! also in KORALW internal commons MOMSET and MOMDEC:
!   COMMON / MOMSET / QEFF1(4),QEFF2(4),SPHUM(4),SPHOT(100,4),NPHOT  
!      effective parameters for Matr.El. only (e-,e+); .. ;photons ;phot. multipl.
!   COMMON / MOMDEC / Q1(4),Q2(4),P1(4),P2(4),P3(4),P4(4)
!      W- ;W+ ;f (W-);\bar f (W-);f (W+);\bar f (W+);
! npar(10)=nevtru = number of accepted UNWEIGHTED events or
!                   number of generated WEIGHTED events     
! npar(11)=nevtot = number of generated events (before rej.)   
!                   (for weighted evts nevtru=nevtot)
! xpar(20)=xsecpb = BEST cross section in picobarns 
!                   / Born for NO bremss. option, 2 ord exp for bremss. 
! xpar(21)=errpb  = error in picobarns 
! xpar(30)=xcrude = crude Born cross section in picobarns  
!       (xcrude* NEVTRU/NEVTOT*WTMAX for UNWEIGHTED evts.)       
! xpar(31)=wtmax  = max wt used for rejection
! **************************    
! For advanced users only:     
! For KEYWGT=1 weighted events are generated and the user should   
! use the weight WTMOD from the common block /WGTALL/.      
! WTMOD is the actual model weight depending on other input params.    
! The other interesting possibility is to use      
!     WT=WTCRUD*WTSET(i) where   
!     WTSET( 1) =   born          
!     WTSET( 2) =   first order           
!     WTSET( 3) =   second order           
! and the corresponding components      
!     WTSET(20) =   First order, beta0 contribution alone   
!     WTSET(21) =   First order, beta1 contribution alone   
!     WTSET(30) =   Second order, beta0 contribution alone   
!     WTSET(31) =   Second order, beta1 contribution alone   
!     WTSET(32) =   Second order, beta2 contribution alone   
! N.B. WTMOD=WTCRUD*WTSET(3)    
!      ( WTMOD=WTCRUD*WTSET(1) for No Bremsstrahlung option )
!   
!     ********************************** 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)          
      DIMENSION  XPAR( *),NPAR( *)      
      COMMON / CGLIB / BLIBK(50000)
      SAVE   / CGLIB / 
      COMMON / MATPAR / PI,CEULER     
      COMMON / PHYPAR / ALFINV,GPICOB     
      COMMON / CMONIT/ AVERWT,ERRELA,NEVTOT,NEVACC,NEVNEG,NEVOVE,NEVZER
      COMMON / MOMSET / QEFF1(4),QEFF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON / MOMDEC / Q1(4),Q2(4),P1(4),P2(4),P3(4),P4(4)
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF    
      COMMON / WEKIN2 / AMAW,GAMMW,GMU,ALPHAW   
      COMMON / WGTGEN /WTMAX,WTMAX4F(20),WTVES,WTYFS,WTSS,WTBWIG,WTBORN
!      
! This common can be everywhere, contains various switches
      COMMON / KeyKey/  KeyRad,KeyPhy,KeyTek,KeyMis,KeyDwm,KeyDwp
! TAUOLA, PHOTOS and JETSET overall switches
      COMMON / LIBRA  / JAK1,JAK2,ITDKRC,IFPHOT,IFHADM,IFHADP
!
      COMMON / WGTALL / WTCRUD,WTMOD,WTSET(100)      
      COMMON / INOUT  / NINP,NOUT   
      COMMON / VVREC  / VVMIN,VVMAX,VV,BETI                   
      COMMON / SSTHE  / S1,S2                 
      COMMON / ANGLES / COSTHE,PHI,COSDE1,PHI1,COSDE2,PHI2
      COMMON / BXFMTS / BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G 
      COMMON / DECAYS / IFLAV(4), AMDEC(4), BR(2), BREL
!      COMMON / DECANG / CT1,CT2
!   -- upspeeding
      COMMON /upspeed/ FACWEL,FACZEL
      SAVE   /upspeed/     
      parameter (NRCH=100)
      common /kanalarz/ fakI(NRCH),ikan,MRCHAN,NRCHAN
      save  /kanalarz/
      CHARACTER*80      BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G 
      SAVE   / CMONIT /,/ MOMSET /,/ WEKING /,/ VVREC /,/ SSTHE /
      SAVE   / WGTGEN /,/ WGTALL /,/ INOUT  /,/ BXFMTS /
      SAVE   / WEKIN2 /,/ DECAYS /,/ MOMDEC /,/ ANGLES /,/ KeyKey/ 
      common /nevik/ nevtru,ifprint
      save nevik
      real*4 rrx
      common /erery/ rrx(14)
      save erery
!-- 4fermion weights 
      DIMENSION wt4f(9)
!-- Vector of random numbers
      DIMENSION drvec(100)
      dimension q12(4),q13(4),q14(4),q23(4),q24(4),q34(4),qtot(4)
      dimension qq1(4),qq2(4),qq3(4),qq4(4),XQ1(4),XQ2(4)
!-- Single precision parameter for Tauola
      REAL POL(4)
!-- Externals
      EXTERNAL RHOSKO,VVDISB
!-- Test DipSwitch KARDMP:  printout on wt over wtmax: 1-on, 0-off
      DATA KARDMP /1/
      SAVE


! ==================================================================
! =====================INITIALIZATION===============================
! ==================================================================
!     *******************
      IF(MODE.EQ.-1) THEN
!     *******************
      
      write(6,*) 'koralw  <-1>'

! Initialization of common blocks
      CALL filexp(xpar,npar)
      write(6,*) 'koralw  <-2>'

!= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = 
! Identificator for THIS Generator 
      idgen = 7 
! Important histo which remembers total x-section 
      CALL Gmonit(  -1, idgen,1d0,1d0,1d0)          
!= = = = = = = = = = = = = = = = = = = = = = = = = = = = = = 

! This is "pointer" for internal monitoring histograms/averages     
      IDYFS = 0

      SVAR=4*ENE**2
      KeyWgt = MOD(KeyTek,10)
      KeyISR = MOD(KeyRad,10)
      Key4f  = MOD(KeyMis,100)/10
      KeySmp = MOD(KeyTek,1000)/100 
!      if(keywgt.eq.1) kardmp=0
      DO i=1,100
        WTSET(i)=0
      ENDDO

!!!!!!!!!!! This should go out to tests !!!!!!
* ============================================================
* Let us keep for KORALW the GLIBK ID-ent range from 2 to 1000 
* ============================================================
!      IF( Key4f .NE. 0 ) THEN
!-- all 4-fermion 
         CALL GMONIT(-1,IDYFS+91,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+92,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+93,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+94,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+95,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+96,0D0,1D0,1D0)  
!      ENDIF
      IF( KeyISR .NE. 0 ) THEN
!-- betas
         CALL GMONIT(-1,IDYFS+71,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+72,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+73,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+74,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+75,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+76,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+77,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+78,0D0,1D0,1D0)  
         CALL GMONIT(-1,IDYFS+80,0D0,1D0,1D0)  ! WTMOD over WTMAX
      ENDIF
!!!!!!!!!!!
 
      write(6,*) 'koralw  <-3>'
!-- Initialization of QED part
      CALL karlud(-1,xcrude,dum2,dum3)     

      IEVACC=0
      NEVTOT=0
      NEVTRU=0
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
! Initialization TAUOLA PHOTOS etc.
!!      IF( KeyWgt .EQ. 0) THEN
         write(6,*) '>>>>>>>> Initialization TAUOLA PHOTOS etc.'
         CALL  inietc(npar(21),npar(22),npar(23),npar(24))
         CALL  inimas
         CALL  iniphx(0.01D0)
         CALL  initdk
         CALL  phoini
!!      ENDIF
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
! Initialization of 4fermion Matrix El. libraries
      IF( Key4f.NE.0 ) THEN
        CALL AMPINW(XPAR,NPAR)
      ENDIF
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      wtmaxi=WTMAX
      wtu=0d0
      write(6,*) 'koralw  <-4>'
   
! ==================================================================
! ========================GENERATION================================
! ==================================================================
!     ***********************
      ELSEIF(MODE.EQ.0) THEN    
!     ***********************

      NEVTRU=NEVTRU+1
 200  CONTINUE
      NEVTOT=NEVTOT+1
cc      CALL KARLUD( 0,DUM1,WTKARL,WTBORN)     
      CALL KARLUD( 0,XCBORN,WTKARL,WTBORN)     
      SPRIM=SVAR*(1-VV)

      WTCRUD   =WTKARL
!---------------------------------------------------------
!--          MODEL weight
!---------------------------------------------------------
!---------------------------------------------------------
!--{{{{{{{{{    This should go to MODEL !!!!!!!
!---------------------------------------------------------
      IF( KeyISR .NE. 0 ) THEN
        wtbe00 =0d0
        wtbe01 =0d0
        wtbe02 =0d0
        wtbe10 =0d0
        wtbe11 =0d0
        wtbe20 =0d0
        IF(WTCRUD .NE. 0d0) THEN
          CALL 
     $      betax(svar,amel,wtbe00,wtbe01,wtbe02,wtbe10,wtbe11,wtbe20)
          wtx0=wtbe00
          wtx1=wtbe01 +wtbe10
          wtx2=wtbe02 +wtbe11 + wtbe20
        ENDIF
      ENDIF

      IF( KeyISR .EQ. 0 ) THEN
! **********************************
!       Principal weight
        WTMOD    =WTCRUD*WTBorn
! **********************************
        WTSET(1) =0d0
        WTSET(2) =0D0
        WTSET(20)=0D0
        WTSET(21)=0D0
      ELSE
! **********************************
!       Principal weight
        WTMOD    =WTCRUD*WTBorn*wtx2
! **********************************
        WTSET(1) =WTBorn*wtx0
        WTSET(2) =WTBorn*wtx1
        WTSET(3) =WTBorn*wtx2
        WTSET(20)=WTBorn*wtbe01
        WTSET(21)=WTBorn*wtbe10
        WTSET(30)=WTBorn*wtbe02
        WTSET(31)=WTBorn*wtbe11
        WTSET(32)=WTBorn*wtbe20
      ENDIF       

!---------------------}}}}}}}}}

!---------------------------------------------------------
!------[[[[[[[[    This should go to Tests !!!!!!!
!---------------------------------------------------------
      WTTOT=WTKARL*WTBORN 
      IF( KeyISR .NE. 0 ) THEN
!-- betas..................
        CALL GMONIT(0,IDYFS+71,wtbe01,  1D0,1D0)        
        CALL GMONIT(0,IDYFS+72,wtbe10,  1D0,1D0)        
        CALL GMONIT(0,IDYFS+73,wtx1,    1D0,1D0)        
        CALL GMONIT(0,IDYFS+74,WTTOT*wtbe01,  1D0,1D0)        
        CALL GMONIT(0,IDYFS+75,WTTOT*wtbe10,  1D0,1D0)        
        CALL GMONIT(0,IDYFS+76,WTTOT*wtx1,    1D0,1D0)        
        CALL GMONIT(0,IDYFS+77,wtx2,    1D0,1D0)        
        CALL GMONIT(0,IDYFS+78,WTTOT*wtx2,    1D0,1D0)        
      ENDIF

!-- printout for big weights
      IF(kardmp.eq.1.and.wtborn.gt.100*wtmax.and.KeySmp.eq.0) THEN
        CTHE1=COS(ANGLE(QEFF1,P1))
        CTHE2=COS(ANGLE(QEFF1,P2))
        CTHE3=COS(ANGLE(QEFF1,P3))
        CTHE4=COS(ANGLE(QEFF1,P4))
!ms        WLAMBD=SPRIM**2+S1**2+S2**2-2*SPRIM*S1-2*SPRIM*S2-2*S1*S2
        wlambd=max(0d0,(sprim-s1-s2)**2 -4*s1*s2)
        T=-(SPRIM-S1-S2-DSQRT(WLAMBD)*COSTHE)/2
        write(6,*) 'wtbor=',real(wtborn),' wttot=',real(wttot)
     @            ,' log t=',real(dlog(-t))
        write(6,*) 'costhe=',real(costhe)
     @            ,' sqrt sprim=',real(dsqrt(sprim))
        write(6,*) 'sqrt s1=',real(sqrt(s1)),' sqrt s2=',real(sqrt(s2))
     @             ,' sqrt(wlambd)/sprim=',real(dsqrt(wlambd)/sprim)
        write(6,*)
     @            'decay c1e-,c2-n,c3n,c4e+',real(cthe1),real(cthe2)
     @             ,real(cthe3),real(cthe4)
        write(6,*) 'keytek',keytek,'keysmp',keysmp
      ENDIF
!------------]]]]]]]]

!====================================================================

!        = = = = = = = = = = = = = = = = = = = = =  
!  Optional Rejection according to principal weight  
!        = = = = = = = = = = = = = = = = = = = = =  

      IF( keywgt.EQ.0 ) THEN
!     = = = = = = = = = = = = = = = = = = = = =
        CALL Gmonit(0,idgen,xcrude,wtmax,1d0)
        IF( WTCRUD.NE.0d0 ) THEN
!---    External 4fermion Matrix El. 
          IF( Key4f.GT.0 ) THEN
            CALL AMPEXT(WTMOD4F,WT4F )
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            KeyZon = MOD(KeyMis,10000)/1000
            KeyWon = MOD(KeyMis,100000)/10000
            if(abs(iflav(1)).eq.11) then
             if(abs(iflav(2)).eq.11) then
               wtmod4f=wtmod4f/FACZEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACZEL
               enddo
             else
               wtmod4f=wtmod4f/FACWEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACWEL
               enddo
             endif
            endif 
            if(abs(iflav(4)).eq.11) then
             if(abs(iflav(3)).eq.11) then
               wtmod4f=wtmod4f/FACZEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACZEL
               enddo
             else
               wtmod4f=wtmod4f/FACWEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACWEL
               enddo
             endif
            endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          ELSE
            WTMOD4F = 1d0
            DO i4f=1,9
              WT4F(i4f)=0d0
            ENDDO
          ENDIF
 
        ELSE

          WTMOD4F = 0d0
          DO i4f=1,9
            WT4F(i4f)=0d0
          ENDDO

        ENDIF
!---
        CALL varran(drvec,1)
        rn = drvec(1)

        IF(wtmod*WTMOD4F.lt.wtmax) THEN 
          WTOVR=0D0
        ELSE
          WTOVR=WTMOD*WTMOD4F
        ENDIF
        CALL gmonit(0,idyfs+80,wtovr,  1d0,1d0)        
!-- 4-fermion monitoring
        CALL GMONIT(0,IDYFS+92,wtmod*wtmod4f,1D0,1D0)  
        CALL GMONIT(0,IDYFS+91,wtmod4f,1D0,1D0)  
        CALL GMONIT(0,IDYFS+93,wtmod  ,1D0,1D0)  
        CALL GMONIT(0,IDYFS+94,wt4f(1),1D0,1D0)  
        CALL GMONIT(0,IDYFS+95,wt4f(2),1D0,1D0)  
        CALL GMONIT(0,IDYFS+96,wtmod*(wtmod4f -1d0),1D0,1D0)  
!-- end 4-fermion monitoring

        IF(wtmod*WTMOD4F .LT. rn*wtmax ) THEN
          GOTO 200
        ENDIF
        WTCRUD   =1D0
        WTMOD    =max(1D0,wtmod*WTMOD4F/wtmax)
        DO i=1,100
           WTSET(i)=0
        ENDDO
      elseIF( keywgt.EQ.2 ) THEN
!     = = = = = = = = = = = = = = = = = = = = =
        CALL Gmonit(0,idgen,xcrude,wtmax,1d0)
        CALL varran(drvec,1)
        rn = drvec(1)
        IF(wtmod.lt.wtmax) THEN 
          WTOVR=0D0
        ELSE
          WTOVR=WTMOD
        ENDIF
        CALL gmonit(0,idyfs+80,wtovr,  1d0,1d0)        
!-- 4-fermion monitoring
        CALL GMONIT(0,IDYFS+92,wtmod*wtmod4f,1D0,1D0)  
        CALL GMONIT(0,IDYFS+91,wtmod4f,1D0,1D0)  
        CALL GMONIT(0,IDYFS+93,wtmod  ,1D0,1D0)  
        CALL GMONIT(0,IDYFS+94,wt4f(1),1D0,1D0)  
        CALL GMONIT(0,IDYFS+95,wt4f(2),1D0,1D0)  
        CALL GMONIT(0,IDYFS+96,wtmod*(wtmod4f -1d0),1D0,1D0)  
!-- end 4-fermion monitoring

        IF(wtmod .LT. rn*wtmax ) THEN
          GOTO 200
        ENDIF
        WTCRUD   =1D0
        WTMOD    =max(1D0,wtmod/wtmax)
        IF( WTCRUD.NE.0d0 ) THEN
!---    External 4fermion Matrix El. 
          IF( Key4f.GT.0 ) THEN
            CALL AMPEXT(WTMOD4F,WT4F )
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            KeyZon = MOD(KeyMis,10000)/1000
            KeyWon = MOD(KeyMis,100000)/10000
            if(abs(iflav(1)).eq.11) then
             if(abs(iflav(2)).eq.11) then
               wtmod4f=wtmod4f/FACZEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACZEL
               enddo
             else
               wtmod4f=wtmod4f/FACWEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACWEL
               enddo
             endif
            endif 
            if(abs(iflav(4)).eq.11) then
             if(abs(iflav(3)).eq.11) then
               wtmod4f=wtmod4f/FACZEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACZEL
               enddo
             else
               wtmod4f=wtmod4f/FACWEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACWEL
               enddo
             endif
            endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          ELSE
            WTMOD4F = 1d0
            DO i4f=1,9
              WT4F(i4f)=0d0
            ENDDO
          ENDIF
 
        ELSE

          WTMOD4F = 0d0
          DO i4f=1,9
            WT4F(i4f)=0d0
          ENDDO

        ENDIF
!---
        DO i=1,100
           WTSET(i)=0
        ENDDO
      ELSEIF( keywgt.EQ.1 ) THEN
!     = = = = = = = = = = = = = =
        IF( WTCRUD.NE.0d0 ) THEN
!---    External 4fermion Matrix El. 
          IF( Key4f.GT.0 ) THEN
            CALL AMPEXT(WTMOD4F,WT4F )
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            KeyZon = MOD(KeyMis,10000)/1000
            KeyWon = MOD(KeyMis,100000)/10000
            if(abs(iflav(1)).eq.11) then
             if(abs(iflav(2)).eq.11) then
               wtmod4f=wtmod4f/FACZEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACZEL
               enddo
             else
               wtmod4f=wtmod4f/FACWEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACWEL
               enddo
             endif
            endif 
            if(abs(iflav(4)).eq.11) then
             if(abs(iflav(3)).eq.11) then
               wtmod4f=wtmod4f/FACZEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACZEL
               enddo
             else
               wtmod4f=wtmod4f/FACWEL
               do k=1,9
                  wt4f(k)=wt4f(k)/FACWEL
               enddo
             endif
            endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          ELSE
            WTMOD4F = 1d0
            DO i4f=1,9
              WT4F(i4f)=0d0
            ENDDO
          ENDIF
 
        ELSE

          WTMOD4F = 0d0
          DO i4f=1,9
            WT4F(i4f)=0d0
          ENDDO

        ENDIF
!---
!-- Weighted events  
        IF(wtmod*WTMOD4F.lt.wtmax) THEN 
          WTOVR=0D0
        ELSE
          WTOVR=WTMOD*WTMOD4F
        ENDIF
        CALL gmonit(0,idyfs+80,wtovr,  1d0,1d0)        
        CALL Gmonit(  0, idgen,      xcrude, wtmax,1d0)
!-- 4-fermion monitoring
        CALL GMONIT(0,IDYFS+92,wtmod*wtmod4f,1D0,1D0)  
        CALL GMONIT(0,IDYFS+91,wtmod4f,1D0,1D0)  
        CALL GMONIT(0,IDYFS+93,wtmod  ,1D0,1D0)  
        CALL GMONIT(0,IDYFS+94,wt4f(1),1D0,1D0)  
        CALL GMONIT(0,IDYFS+95,wt4f(2),1D0,1D0)  
        CALL GMONIT(0,IDYFS+96,wtmod*(wtmod4f -1d0),1D0,1D0)  
!-- end 4-fermion monitoring

      ELSE
        WRITE(6,*)'KORALW==>WRONG KEYWGT=',KEYWGT
        STOP
      ENDIF
!     = = = 
!          if (nevtru.eq.3320) goto 777
 !         if (nevtru.gt.3300) write(*,*) nevtru,' ',wtmod,wtmod4f 
          if(KARDMP.eq.1.and.wtovr/wtmax.gt.1.0d0) then
 777         continue
          wtu=max(wtovr/wtmax,wtu)
          do k=1,4
          q12(k)=p1(k)+p2(k)
          q13(k)=p1(k)+p3(k)
          q14(k)=p1(k)+p4(k)
          q23(k)=p2(k)+p3(k)
          q24(k)=p2(k)+p4(k)
          q34(k)=p3(k)+p4(k)
          qtot(k)=p1(k)+p2(k)+p3(k)+p4(k)
          enddo
          xm12=sqrt(dmas2(q12))
          xm13=sqrt(dmas2(q13))
          xm14=sqrt(dmas2(q14))
          xm23=sqrt(dmas2(q23))
          xm24=sqrt(dmas2(q24))
          xm34=sqrt(dmas2(q34))


          do k=1,4
          qq1(k)=      p2(k)+p3(k)+p4(k)
          qq2(k)=p1(k)      +p3(k)+p4(k)
          qq3(k)=p1(k)+p2(k)      +p4(k)
          qq4(k)=p1(k)+p2(k)+p3(k)
          enddo
          xm1=sqrt(dmas2(qq1))
          xm2=sqrt(dmas2(qq2))
          xm3=sqrt(dmas2(qq3))
          xm4=sqrt(dmas2(qq4))
          xmtot=sqrt(dmas2(qtot))

      DO I=1,4
       XQ1(I)=QEFF1(I)
       XQ2(I)=QEFF2(I)
      ENDDO

!        do k=1,3
!         xq1(k)=xq1(k)*(xq1(4)-0.5*amel**2/xq1(4))/xq1(4)
!         xq2(k)=xq2(k)*(xq2(4)-0.5*amel**2/xq2(4))/xq2(4)
!        enddo
          do k=1,4
          qq1(k)=xq1(k)      -p1(k)
          qq2(k)=xq2(k)      -p2(k)
          qq3(k)=xq1(k)      -p3(k) 
          qq4(k)=xq2(k)      -p4(k)
          enddo
          Ym1=sqrt(-dmas2(qq1))
          Ym2=sqrt(-dmas2(qq2))
          Ym3=sqrt(-dmas2(qq3))
          Ym4=sqrt(-dmas2(qq4))
          i6=6
          write(i6,*) '====OVERVEIGHTED EVT. NR: NEVTRU=',NEVTRU,'====='
          write(i6,*) '===generated with channel: ikan=',ikan,'========'
          write(i6,*) 'Note: this event should be outside your detector'
          write(i6,*) 'if not, increase wtmax (?) and/or check manual ?'
          write(i6,*) 'final state: ',IFLAV
          write(i6,*) 'wtu= wtmod*wtmod4f/wtmax=',wtovr/wtmax,
     $                ' wtmod4f=',wtmod4f
          write(i6,*) 'wtu_max=                 ',wtu,'    ... so far'
          write(i6,*) '-----------'
          write(i6,*) 'e-prim=',xmtot,'  wtmod4f=',wtmod4f
           write(i6,*) 'active entries from',mrchan,'to',nrchan,';'
          do k=1,nrchan/5+1
           write(i6,*)  'jacobians(',5*k-4,'--',5*k,')=',
     $     faki(5*k-4),faki(5*k-3),faki(5*k-2),faki(5*k-1),faki(5*k) 
          enddo
          write(i6,*) '-----------'
          write(i6,*) ' m12=',xm12,' m13=',xm13,' m14=',xm14
          write(i6,*) ' m23=',xm23,' m24=',xm24,' m34=',xm34
          write(i6,*) '-----------'
          write(i6,*) ' m-1=',xm1,' m-2=',xm2,' m-3=',xm3,' m-4=',xm4
          write(i6,*) '-----------'
          write(i6,*) ' T-1=',Ym1,' T-2=',Ym2,' T-3=',Ym3,' T-4=',Ym4
          write(i6,*) '-----------'
          write(i6,*) 'qtot' ,qtot
          write(i6,*) 'qcms',xq1(1)+xq2(1),xq1(2)+xq2(2),
     $                       xq1(3)+xq2(3),xq1(4)+xq2(4)
          write(i6,*) '-----------------------'
          write(i6,*) '  QEFF1 ' ,QEFF1
          write(i6,*) 'M-QEFF1 ' ,dmas2(qeff1),' ',sqrt(dmas2(xq1))
          write(i6,*) '-----------------------'
          write(i6,*) '  QEFF2 ',QEFF2
          write(i6,*) 'M-QEFF2 ' ,dmas2(qeff2),' ',sqrt(dmas2(xq2))
          write(i6,*) '-----------------------'
          write(i6,*) '-----------------------'
          write(i6,*) 'p1= ',p1
          write(i6,*) '-----------'
          write(i6,*) 'p2= ',p2
          write(i6,*) '-----------'
          write(i6,*) 'p3= ', p3
          write(i6,*) '-----------'
          write(i6,*) 'p4= ',p4
          write(i6,*) '-----------'
          call dumpw(i6)
         do k=1,14
            write(i6,*) '       RRR(',k,')=',RRX(k)
         enddo

!          write(*,*) '=====amp4f here is problem==============='
!          write(*,*) '=====one of inv is too small============='
!          do i=1,6
!          do j=1,6
!           write(*,*) 'i=',i,' j=',j,' pp=',pp(i,j)
!          enddo
!          enddo

          endif
      WTSET(40)=WTMOD4F
      DO I4F=1,9
        WTSET(40+I4F)=WT4F(I4F)
      ENDDO

!-- printout for big weights
      wtbor=wtborn ! this is technical print, commented out
      IF(KARDMP.eq.-1.and.wtmod4f*wtbor.gt.wtmaxi) THEN
        wtmaxi=wtmod4f*wtbor
        CTHE1=COS(ANGLE(QEFF1,P1))
        CTHE2=COS(ANGLE(QEFF1,P2))
        CTHE3=COS(ANGLE(QEFF1,P3))
        CTHE4=COS(ANGLE(QEFF1,P4))
!ms        WLAMBD=SPRIM**2+S1**2+S2**2-2*SPRIM*S1-2*SPRIM*S2-2*S1*S2
        wlambd=max(0d0,(sprim-s1-s2)**2 -4*s1*s2)
        T=-(SPRIM-S1-S2-DSQRT(WLAMBD)*COSTHE)/2
        write(6,*) 'channel ',IFLAV(4)
        write(6,*) 'wtmod4f/max=',real(wtmod4f/wtmaxi)
     @      ,'wtborn*wtmod4f=',real(wtmod4f*wtbor)
        write(6,*) ' wt4f(1)=',real(wt4f(1))
        write(6,*) 'wtbor=',real(wtborn),' wttot=',real(wttot)
     @            ,' log t=',real(dlog(-t))
        write(6,*) 'costhe=',real(costhe)
     @            ,' sqrt sprim=',real(dsqrt(sprim))
        write(6,*) 'sqrt s1=',real(sqrt(s1)),' sqrt s2=',real(sqrt(s2))
     @             ,' sqrt(wlambd)/sprim=',real(dsqrt(wlambd)/sprim)
        write(6,*)
     @            'decay c1e-,c2-n,c3n,c4e+',real(cthe1),real(cthe2)
     @             ,real(cthe3),real(cthe4)
        write(6,*) '        ============================'
      ENDIF
!--------------------------------------------------------------------

! CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      IF( wtcrud .NE. 0d0) THEN
! CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
! TOHEP sets into HEPEVT all generated particles.
!       it decays taus and generates bremsstrahlung
!       in tau and W decays.
         CALL TOHEP
! and TOHAD moves to LUND format.
! It hadronizes whatever requires.
         CALL TOHAD(IFHADM,IFHADP)
      ELSE
! Some routine to set HEPEVT to 0 should be here <<<<<============
        continue
! CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      ENDIF
! CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!--------------------------------------------------------------------

! CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      IF((NEVTRU.LE.10. OR. NEVTRU.EQ.2000000).AND.WTKARL.GT.0D0)THEN
c         write(   6,*) 'wtborn=',wtborn
c         write(nout,*) 'wtborn=',wtborn
c         CALL DUMPL(6,Q1,Q2,P1,P2,P3,P4,QEFF1,QEFF2,sphot,nphot)
         CALL DUMPW(6)
         CALL DUMPW(NOUT)
      ENDIF

!      IF(wtkarl.ne.0) THEN
!        CALL EXCAL(SVAR,SPRIM,S1,S2,XMATR) !COMP. WITH EXCALIBUR
!      ENDIF
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&


! ==================================================================
! ====================POSTgeneration================================
! ==================================================================
!     *************************
      ELSEIF( mode .EQ .1) THEN     
!     *************************

      CALL karlud(1,xcrude,xcvesk,dumm1)       

      IF( KeyISR .NE. 0 ) THEN
!-- betas

!-- beta0 1ord
        CALL GMONIT(1,IDYFS+71,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXOPE)         
        WRITE(NOUT,BXTXT) '         KORALW  FINAL  REPORT '
        WRITE(NOUT,BXTXT) '               window B        '
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXL1I) NEVTOT, 'total no of events '  ,'NEVTOT','B0a'
        WRITE(NOUT,BXL1I) NEVTRU,  'accepted events '    ,'NEVTRU','B0b'
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) ' Beta0  1st Order contrib. to Total Xsect.'
        WRITE(NOUT,BXL1I) NEVNEG,  'WTbe01<0  events '    ,'NEVNEG','B0'
        WRITE(NOUT,BXL2F) WTKABO,ERKABO
     $                             ,'<WTbe01>virt rel err','WTKABO','B1'
        CALL GMONIT(1,IDYFS+74,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        XSKB   = XCRUDE*WTKABO
        ERKB   = XSKB*ERKABO
        WRITE(NOUT,BXL2F) WTKABO,ERKABO
     $                           ,'<WTSET(20)*WTCRUD>virt','WTKABO','B2'
        WRITE(NOUT,BXL2F) XSKB,ERKB, 'sigma (born*bevi01)','XSKABO','B3'

!-- beta1 1ord
        CALL GMONIT(1,IDYFS+72,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) ' Beta1  1st Order contrib. to Total Xsect.'
        WRITE(NOUT,BXL1I) NEVNEG,  'WTbe10<0  events '    ,'NEVNEG','B4'
        WRITE(NOUT,BXL2F) WTKABO,ERKABO
     $                             ,'<WTbe10>real rel err','WTKABO','B5'
        CALL GMONIT(1,IDYFS+75,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        XSKB   = XCRUDE*WTKABO
        ERKB   = XSKB*ERKABO
        WRITE(NOUT,BXL2F) XSKB,ERKB, 'sigma (born*bere10)','XSKABO','B6'
        WRITE(NOUT,BXCLO)  

!-- beta0+beta1 1ord
        CALL GMONIT(1,IDYFS+73,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXOPE)         
        WRITE(NOUT,BXTXT) '         KORALW  FINAL  REPORT '
        WRITE(NOUT,BXTXT) '               window C        '
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) '        1st Order  Total Xsect.'
        WRITE(NOUT,BXL1I) NEVNEG,  'WTbe0+1<0  events '   ,'NEVNEG','C0'
        WRITE(NOUT,BXL2F) 
     $          WTKABO,ERKABO,'<WTbe0+1> 1ord rel err','WTKABO','C1'
        CALL GMONIT(1,IDYFS+76,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        XSKB   = XCRUDE*WTKABO
        ERKB   = XSKB*ERKABO
        WRITE(NOUT,BXL2F)XSKB,ERKB,'sigma O(alpha)'       ,'XSKABO','C2'
!-- beta0+beta1+beta2 2ord
        CALL GMONIT(1,IDYFS+77,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) '        2nd Order  Total Xsect.'
        WRITE(NOUT,BXL1I) NEVNEG,'WTbe0+1+2<0  events '   ,'NEVNEG','C3'
        WRITE(NOUT,BXL2F) 
     $          WTKABO,ERKABO,'<WTbe0+1+2> 2ord rel err'  ,'WTKABO','C4'
        CALL GMONIT(1,IDYFS+78,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXL2F) 
     $          WTKABO,ERKABO,'<WTmod> 2ord rel err'  ,'WTKABO','C4a'
        XSKB   = XCRUDE*WTKABO
        ERKB   = XSKB*ERKABO
        WRITE(NOUT,BXL2F)XSKB,ERKB,'sigma O(alpha**2)'    ,'XSKABO','C5'

!-- beta0+beta1+beta2 2ord OVER wtmax
        CALL GMONIT(1,IDYFS+80,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        XSKB   = XCRUDE*WTKABO
        ERKB   = XSKB*ERKABO
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) ' Best Order (WTMOD) Total Xsect. OVER wtmax'
        WRITE(NOUT,BXL1I) NEVTOT, 'total no of events '  ,'NEVTOT','B0a'
        WRITE(NOUT,BXL1I) NEVTRU,  'accepted events '    ,'NEVTRU','B0b'
        WRITE(NOUT,BXL1I) NEVACC,  'no of ev OVER wtmax','NEVACC','C6'
        WRITE(NOUT,BXL2F)XSKB,ERKB,'sigma MODEL OVER   ','XSKABO','C7'
        WRITE(NOUT,BXCLO)         
!-- end betas
      ENDIF     

!-- 4 fermion matrix el. 
      IF( Key4f .NE. 0 ) THEN
!-- 4-fermion monitoring
        WRITE(NOUT,BXOPE)         
        WRITE(NOUT,BXTXT) '         KORALW  FINAL  REPORT '
        WRITE(NOUT,BXTXT) '               window D        '
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) '     complete 4-fermion process'
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) '    Best Ord. W-pair Total Xsect.'
        CALL GMONIT(1,IDYFS+93,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXL2F) 
     $          WTKABO,ERKABO,'WTmod (A7 or c4)'         ,'WTKABO','D1'
        XSKBs   = XCRUDE*WTKABO
        ERKBs   = XSKBs*ERKABO
        WRITE(NOUT,BXL2F)XSKBs,ERKBs,'sigma WTmod, best','XSKABO','D2'
        WRITE(NOUT,BXTXT) '                               '
        WRITE(NOUT,BXTXT) '    Best Ord. 4-fermion Total Xsect. '
        CALL GMONIT(1,IDYFS+91,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXL1I) NEVNEG,'WTmod4f<0  events'      ,'NEVNEG','D3'
        WRITE(NOUT,BXL2G) 
     $          WTKABO,ERKABO,'WTmod4f rel err'           ,'WTKABO','D4'
        CALL GMONIT(1,IDYFS+92,DUMM1,DUMM2,DUMM3)   
        WTKABO = AVERWT      
        ERKABO = ERRELA 
        WRITE(NOUT,BXL2F) 
     $          WTKABO,ERKABO,'WTmod*WTmod4f rel err'     ,'WTKABO','D5'
        XSKBb   = XCRUDE*WTKABO
        ERKBb   = XSKBb*ERKABO
        WRITE(NOUT,BXL2F)XSKBb,ERKBb,'sigma 4fermion,best','XSKABO','D6'
        StoB = 1- XSKBs/XSKBb
        StoBer = dsqrt( (ERKBs*XSKBb)**2 +(ERKBb*XSKBs)**2 ) / XSKBb**2
        WRITE(NOUT,BXL2F)StoB,StoBer,'sigma 1-Wpair/4ferm','1-D2/6','D7'
        CALL GMONIT(1,IDYFS+96,DUMM1,DUMM2,DUMM3) 
        WTKABO = AVERWT 
        ERKABO = ERRELA 
        XSKBd   = XCRUDE*WTKABO
        ERKBd   = XSKBd*ERKABO
        StoB = XSKBd/XSKBb
        StoBer = dsqrt( (ERKBd*XSKBb)**2 +(ERKBb*XSKBd)**2 ) / XSKBb**2
        WRITE(NOUT,BXL2F)StoB,StoBer,'sigma 1-Wpair/4ferm','XSKABO','D8'
        WRITE(NOUT,BXCLO)         

      ENDIF

!-- best xsection, total, for output purposes only!
      IF( Key4f .EQ. 0 ) THEN
!  No 4fermion introduced 
        IF( KeyISR .eq. 0) THEN
          CALL GMONIT(1,IDYFS+58,DUMM1,DUMM2,DUMM3)   
        ELSE
          CALL GMONIT(1,IDYFS+78,DUMM1,DUMM2,DUMM3) 
        ENDIF  
      ELSEIF( Key4f .NE. 0 ) THEN
!  4fermion introduced 
        CALL GMONIT(1,IDYFS+92,DUMM1,DUMM2,DUMM3) 
      ENDIF

      XSBEST  = XCRUDE*AVERWT 
      ERBEST  = XSBEST*ERRELA 

      NPAR(10)=NEVTRU
      NPAR(11)=NEVTOT
      XPAR(20)=XSBEST
      XPAR(21)=ERBEST
      XPAR(30)=XCRUDE !!!!! <==== to be changed
!!!!!!!!!!!!!!!!!!!!!! WARNING !!!!!!!!
ccc      XPAR(31)=WTMAX
      XPAR(31)=WTMAX
      IF(keywgt.eq.0 .OR. keywgt.eq.2) 
ccc     $ XPAR(30)=XPAR(30) *NEVTRU/NEVTOT*WTMAX !!!!! <====
     $ XPAR(30)=XPAR(30) *NEVTRU/NEVTOT*WTMAX !!!!! <====
!!!!!!!!!!!!!!!!!!!!!! WARNING !!!!!!!!

C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
! Final printouts of TAUOLA
      KeyWgt = MOD(KeyTek,10)
      IF( KeyWgt .EQ. 0) THEN
         CALL DEXAY(100,POL)
      ENDIF
C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


!     **********************
      ELSEIF(MODE.EQ.2) THEN     
!     **********************
      CALL KARLUD(MODE,DUM1,XSBORN,ERBORN)       
!     ****
      ELSE          
!     ****
      WRITE(NOUT,*) '===>KORALW: WRONG MODE'                
      STOP          
!     *****        
      ENDIF         
!     *****        
      END                 






